@@ -131,7 +131,7 @@ unsigned char *encode_utf_8(int u)
 		utf_buffer[2] = 0x80 | ((u >> 12) & 0x3f),
 		utf_buffer[3] = 0x80 | ((u >> 6) & 0x3f),
 		utf_buffer[4] = 0x80 | (u & 0x3f);
-	else	utf_buffer[0] = 0xfc | ((u >> 30) & 0x0f),
+	else	utf_buffer[0] = 0xfc | ((u >> 30) & 0x01),
 		utf_buffer[1] = 0x80 | ((u >> 24) & 0x3f),
 		utf_buffer[2] = 0x80 | ((u >> 18) & 0x3f),
 		utf_buffer[3] = 0x80 | ((u >> 12) & 0x3f),
@@ -168,9 +168,11 @@ void add_utf_8(struct conv_table *ct, int u, unsigned char *str)
 }
 
 struct conv_table table[256];
+static int first = 1;
 
 void free_conv_table()
 {
+	if (first) memset(table, 0, sizeof(struct conv_table) * 256), first = 0;
 	new_translation_table(table);
 	mem_free(no_str), no_str = NULL;
 }
@@ -180,7 +182,6 @@ struct conv_table *get_translation_table(int from, int to)
 	int i;
 	static int lfr = -1;
 	static int lto = -1;
-	static int first = 1;
 	if (first) memset(table, 0, sizeof(struct conv_table) * 256), first = 0;
 	if (/*from == to ||*/ from == -1 || to == -1) return NULL;
 	if (from == lfr && to == lto) return table;
