@@ -106,6 +106,27 @@ int get_pasv_socket(struct connection *c, int cc, int *sock, unsigned char *port
 	return 0;
 }
 
+#ifdef HAVE_SSL
+void ssl_want_read(struct connection *c)
+{
+	struct conn_info *b = c->buffer;
+
+	if (c->no_tsl) c->ssl->options |= SSL_OP_NO_TLSv1;
+	switch (SSL_get_error(c->ssl, SSL_connect(c->ssl))) {
+		case SSL_ERROR_NONE:
+			c->buffer = NULL;
+			b->func(c);
+			mem_free(b);
+		case SSL_ERROR_WANT_READ:
+			break;
+		default:
+			c->no_tsl++;
+			setcstate(c, S_SSL_ERROR);
+			retry_connection(c);
+	}
+}
+#endif
+
 void dns_found(struct connection *c, int state)
 {
 	int s;
@@ -135,6 +156,25 @@ void dns_found(struct connection *c, int state)
 		set_handlers(s, NULL, (void(*)(void *))connected, (void(*)(void *))exception, c);
 		setcstate(c, S_CONN);
 	} else {
+#ifdef HAVE_SSL
+		if(c->ssl) {
+			c->ssl = getSSL();
+			SSL_set_fd(c->ssl,s);
+			if (c->no_tsl) c->ssl->options |= SSL_OP_NO_TLSv1;
+			switch (SSL_get_error(c->ssl, SSL_connect(c->ssl))) {
+				case SSL_ERROR_WANT_READ:
+					set_handlers(s, (void(*)(void *))ssl_want_read, NULL, (void(*)(void *))exception, c);
+					return;
+				case SSL_ERROR_NONE:
+					break;
+				default:
+					c->no_tsl++;
+					setcstate(c, S_SSL_ERROR);
+					retry_connection(c);
+					return;
+			}
+		}
+#endif
 		c->buffer = NULL;
 		b->func(c);
 		mem_free(b);
@@ -147,7 +187,6 @@ void connected(struct connection *c)
 	void (*func)(struct connection *) = b->func;
 	int err = 0;
 	int len = sizeof(int);
-	c->buffer = NULL;
 	/*if (!connect(*b->sock, (struct sockaddr *)&b->sa, sizeof b->sa) || errno == EISCONN) {
 		mem_free(b);
 		func(c);
@@ -162,9 +201,31 @@ void connected(struct connection *c)
 		}
 	if (err >= 10000) err -= 10000;	/* Why does EMX return so large values? */
 	bla:
-	mem_free(b);
 	if (err > 0) setcstate(c, -err), retry_connection(c);
-	else func(c);
+	else {
+#ifdef HAVE_SSL
+		if(c->ssl) {
+			c->ssl = getSSL();
+			SSL_set_fd(c->ssl, *b->sock);
+			if (c->no_tsl) c->ssl->options |= SSL_OP_NO_TLSv1;
+			switch (SSL_get_error(c->ssl, SSL_connect(c->ssl))) {
+				case SSL_ERROR_WANT_READ:
+					set_handlers(*b->sock, (void(*)(void *))ssl_want_read, NULL, (void(*)(void *))exception, c);
+					return;
+				case SSL_ERROR_NONE:
+					break;
+				default:
+					c->no_tsl++;
+					setcstate(c, S_SSL_ERROR);
+					retry_connection(c);
+					return;
+			}
+		}
+#endif
+		c->buffer = NULL;
+		mem_free(b);
+		func(c);
+	}
 }
 
 struct write_buffer {
@@ -188,11 +249,25 @@ void write_select(struct connection *c)
 	/*printf("ws: %d\n",wb->len-wb->pos);
 	for (wr = wb->pos; wr < wb->len; wr++) printf("%c", wb->data[wr]);
 	printf("-\n");*/
-	if ((wr = write(wb->sock, wb->data + wb->pos, wb->len - wb->pos)) <= 0) {
-		setcstate(c, wr ? -errno : S_CANT_WRITE);
-		retry_connection(c);
-		return;
-	}
+
+#ifdef HAVE_SSL
+	if(c->ssl) {
+		if ((wr = SSL_write(c->ssl, wb->data + wb->pos, wb->len - wb->pos)) <= 0) {
+			if (SSL_get_error(c->ssl, wr) != SSL_ERROR_WANT_WRITE) {
+				setcstate(c, wr ? -errno : S_CANT_WRITE);
+				retry_connection(c);
+				return;
+			}
+			else return;
+		}
+	} else
+#endif
+		if ((wr = write(wb->sock, wb->data + wb->pos, wb->len - wb->pos)) <= 0) {
+			setcstate(c, wr ? -errno : S_CANT_WRITE);
+			retry_connection(c);
+			return;
+		}
+
 	/*printf("wr: %d\n", wr);*/
 	if ((wb->pos += wr) == wb->len) {
 		void (*f)(struct connection *) = wb->done;
@@ -241,17 +316,37 @@ void read_select(struct connection *c)
 		return;
 	}
 	c->buffer = rb;
-	if ((rd = read(rb->sock, rb->data + rb->len, READ_SIZE)) <= 0) {
-		if (rb->close && !rd) {
-			rb->close = 2;
-			rb->done(c, rb);
+
+#ifdef HAVE_SSL
+	if(c->ssl) {
+		if ((rd = SSL_read(c->ssl, rb->data + rb->len, READ_SIZE)) <= 0) {
+			if (SSL_get_error(c->ssl, rd) == SSL_ERROR_WANT_READ) {
+				read_from_socket(c, rb->sock, rb, rb->done);
+				return;
+			}
+			if (rb->close && !rd) {
+				rb->close = 2;
+				rb->done(c, rb);
+				return;
+			}
+			setcstate(c, rd ? -errno : S_CANT_READ);
+			/*mem_free(rb);*/
+			retry_connection(c);
+			return;
+		}
+	} else
+#endif
+		if ((rd = read(rb->sock, rb->data + rb->len, READ_SIZE)) <= 0) {
+			if (rb->close && !rd) {
+				rb->close = 2;
+				rb->done(c, rb);
+				return;
+			}
+			setcstate(c, rd ? -errno : S_CANT_READ);
+			/*mem_free(rb);*/
+			retry_connection(c);
 			return;
 		}
-		setcstate(c, rd ? -errno : S_CANT_READ);
-		/*mem_free(rb);*/
-		retry_connection(c);
-		return;
-	}
 	log_data(rb->data + rb->len, rd);
 	rb->len += rd;
 	rb->done(c, rb);
