@@ -14,8 +14,6 @@ struct ftp_connection_info {
 	unsigned char ftp_buffer[FTP_BUF];
 };
 
-unsigned char *default_anon_pass = "somebody@host.domain";
-
 void ftp_login(struct connection *);
 void ftp_logged(struct connection *);
 void ftp_got_reply(struct connection *, struct read_buffer *);
@@ -101,9 +99,9 @@ void ftp_login(struct connection *c)
 		mem_free(login);
 		return;
 	}
-	setcstate(c, S_SENT);
 	write_to_socket(c, c->sock1, login, strlen(login), ftp_logged);
 	mem_free(login);
+	setcstate(c, S_SENT);
 }
 
 void ftp_logged(struct connection *c)
@@ -225,7 +223,6 @@ int add_file_cmd_to_str(unsigned char **s, int *l, struct connection *c)
 void ftp_send_retr_req(struct connection *c)
 {
 	unsigned char *login;
-	unsigned char *u;
 	int logl = 0;
 	if (!(login = init_str())) {
 		setcstate(c, S_OUT_OF_MEM);
@@ -236,9 +233,9 @@ void ftp_send_retr_req(struct connection *c)
 		mem_free(login);
 		return;
 	}
-	setcstate(c, S_SENT);
 	write_to_socket(c, c->sock1, login, strlen(login), ftp_retr_1);
 	mem_free(login);
+	setcstate(c, S_SENT);
 }
 
 void ftp_retr_1(struct connection *c)
@@ -252,12 +249,11 @@ void ftp_retr_file(struct connection *c, struct read_buffer *rb)
 {
 	int g;
 	struct ftp_connection_info *inf = c->info;
-	setcstate(c, S_GETH);
 	rep:
 	if (inf->pending_commands > 1) {
 		g = get_ftp_response(c, rb, 0);
 		if (g == -1) { setcstate(c, S_FTP_ERROR); abort_connection(c); return; }
-		if (!g) { read_from_socket(c, c->sock1, rb, ftp_retr_file); return; }
+		if (!g) { read_from_socket(c, c->sock1, rb, ftp_retr_file); setcstate(c, S_GETH); return; }
 		inf->pending_commands--;
 		switch (inf->opc - inf->pending_commands) {
 			case 1:		/* TYPE */
@@ -275,7 +271,7 @@ void ftp_retr_file(struct connection *c, struct read_buffer *rb)
 		internal("WHAT???");
 	}
 	g = get_ftp_response(c, rb, 2);
-	if (!g) { read_from_socket(c, c->sock1, rb, ftp_retr_file); return; }
+	if (!g) { read_from_socket(c, c->sock1, rb, ftp_retr_file); setcstate(c, S_GETH); return; }
 	if (g >= 100 && g < 200) {
 		int est;
 		unsigned char *d = rb->data;
@@ -291,7 +287,7 @@ void ftp_retr_file(struct connection *c, struct read_buffer *rb)
 		if (i + 4 > rb->len) goto nol;
 		if (casecmp(&d[i], "byte", 4)) goto nol;
 		est = strtol(&d[p], NULL, 10);
-		if (est) c->est_length = est;
+		if (est && !c->from) c->est_length = est; /* !!! FIXME: when I add appedning to downloaded file */
 		nol:;
 	}
 	set_handlers(c->sock2, (void (*)(void *))got_something_from_data_connection, NULL, NULL, c);
@@ -304,9 +300,9 @@ void ftp_got_final_response(struct connection *c, struct read_buffer *rb)
 	struct ftp_connection_info *inf = c->info;
 	int g = get_ftp_response(c, rb, 0);
 	if (g == -1) { setcstate(c, S_FTP_ERROR); abort_connection(c); return; }
-	if (!g) { rep:read_from_socket(c, c->sock1, rb, ftp_got_final_response); return; }
+	if (!g) { read_from_socket(c, c->sock1, rb, ftp_got_final_response); if (c->state != S_TRANS) setcstate(c, S_GETH); return; }
 	if (g == 550) {
-		if (get_cache_entry(c->url, &c->cache)) {
+		if (!c->cache && get_cache_entry(c->url, &c->cache)) {
 			setcstate(c, S_OUT_OF_MEM);
 			abort_connection(c);
 			return;
@@ -315,6 +311,7 @@ void ftp_got_final_response(struct connection *c, struct read_buffer *rb)
 		c->cache->redirect = stracpy(c->url);
 		c->cache->redirect_get = 1;
 		add_to_strn(&c->cache->redirect, "/");
+		c->cache->incomplete = 0;
 		/*setcstate(c, S_FTP_NO_FILE);*/
 		setcstate(c, S_OK);
 		abort_connection(c);
@@ -324,7 +321,10 @@ void ftp_got_final_response(struct connection *c, struct read_buffer *rb)
 	if (inf->conn_st == 2) {
 		setcstate(c, S_OK);
 		ftp_end_request(c);
-	} else inf->conn_st = 1;
+	} else {
+		inf->conn_st = 1;
+		if (c->state != S_TRANS) setcstate(c, S_GETH);
+	}
 }
 
 #define accept_char(x) ((upcase((x)) >= 'A' && upcase((x)) <= 'Z') || ((x) >= '0' && (x) <= '9') || (x) == ' ' || (x) == '-' || (x) == '_' || (x) == '.' || (x) == ':' || (x) == ';')
@@ -363,22 +363,28 @@ int ftp_process_dirlist(struct cache_entry *ce, int *pos, int *d, unsigned char
 	pl:
 	str = init_str();
 	sl = 0;
-	add_to_str(&str, &sl, "   ");
-	if (*d && *d < p && WHITECHAR(buf[p - 1])) {
+	/*add_to_str(&str, &sl, "   ");*/
+	if (*d && *d < p && WHITECHAR(buf[*d - 1])) {
+		int ee;
 		ppp:
+		for (ee = *d; ee <= p - 4; ee++)
+			if (!memcmp(buf + ee, " -> ", 4)) goto syml;
+		ee = p;
+		syml:
 		add_conv_str(&str, &sl, buf, *d);
 		add_to_str(&str, &sl, "<a href=\"");
-		add_conv_str(&str, &sl, buf + *d, p - *d);
+		add_conv_str(&str, &sl, buf + *d, ee - *d);
 		if (buf[0] == 'd') add_chr_to_str(&str, &sl, '/');
 		add_to_str(&str, &sl, "\">");
-		add_conv_str(&str, &sl, buf + *d, p - *d);
+		add_conv_str(&str, &sl, buf + *d, ee - *d);
 		add_to_str(&str, &sl, "</a>");
+		add_conv_str(&str, &sl, buf + ee, p - ee);
 	} else {
 		int pp;
 		if (p > 5 && !casecmp(buf, "total", 5)) goto raw;
 		for (pp = p - 1; pp >= 0; pp--) if (!WHITECHAR(buf[pp])) break;
 		if (pp < 0) goto raw;
-		for (; pp >= 0; pp--) if (WHITECHAR(buf[pp])) break;
+		for (; pp >= 0; pp--) if (WHITECHAR(buf[pp]) && (pp < 3 || memcmp(buf + pp - 3, " -> ", 4)) && (pp > p - 4 || memcmp(buf + pp, " -> ", 4))) break;
 		*d = pp + 1;
 		goto ppp;
 		raw:
@@ -406,8 +412,7 @@ void got_something_from_data_connection(struct connection *c)
 		set_handlers(ns, (void (*)(void *))got_something_from_data_connection, NULL, NULL, c);
 		return;
 	}
-	setcstate(c, S_TRANS);
-	if (get_cache_entry(c->url, &c->cache)) {
+	if (!c->cache && get_cache_entry(c->url, &c->cache)) {
 		setcstate(c, S_OUT_OF_MEM);
 		abort_connection(c);
 		return;
@@ -448,6 +453,7 @@ void got_something_from_data_connection(struct connection *c)
 			memmove(inf->ftp_buffer, inf->ftp_buffer + m, inf->buf_pos + l - m);
 			inf->buf_pos += l - m;
 		}
+		setcstate(c, S_TRANS);
 		return;
 	}
 	ftp_process_dirlist(c->cache, &c->from, &inf->dpos, inf->ftp_buffer, inf->buf_pos, 1, &c->tries);
@@ -456,14 +462,17 @@ void got_something_from_data_connection(struct connection *c)
 	if (inf->conn_st == 1) {
 		setcstate(c, S_OK);
 		ftp_end_request(c);
-	} else inf->conn_st = 2;
+	} else {
+		inf->conn_st = 2;
+		setcstate(c, S_TRANS);
+	}
 }
 
 void ftp_end_request(struct connection *c)
 {
 	if (c->state == S_OK) {
 		if (c->cache) {
-			truncate_entry(c->cache, c->from);
+			truncate_entry(c->cache, c->from, 1);
 			c->cache->incomplete = 0;
 		}
 	}
