@@ -84,7 +84,11 @@ unsigned char *get_attr_val(unsigned char *e, unsigned char *name)
 		a:
 		e++;
 		while (*e != uu) {
-			if (!f && *e && *e != 10) {
+			if (!*e) {
+				mem_free(a);
+				return NULL;
+			}
+			if (!f && *e != 10) {
 				if (*e != 9 && *e != 13) add_chr(a, l, *e);
 				else add_chr(a, l, ' ');
 			}
@@ -124,11 +128,13 @@ int has_attr(unsigned char *e, unsigned char *name)
 
 unsigned char *get_url_val(unsigned char *e, unsigned char *name)
 {
+	int n = 0;
 	unsigned char *p, *q, *pp;
 	if (!(pp = get_attr_val(e, name))) return NULL;
 	p = pp; q = pp;
 	while (1) {
-		if ((*p = *q) != ' ') p++;
+		if (*p == '#') n = 1;
+		if ((*p = *q) != ' ' || n) p++;
 		if (!*q) break;
 		q++;
 	}
@@ -955,10 +961,10 @@ void html_li(unsigned char *a)
 {
 	/*kill_until(0, "", "UL", "OL", NULL);*/
 	if (!par_format.list_number) {
-		char x[2] = "* ";
+		char x[7] = "*&nbsp;";
 		if ((par_format.flags & P_LISTMASK) == P_O) x[0] = 'o';
 		if ((par_format.flags & P_LISTMASK) == P_PLUS) x[0] = '+';
-		put_chrs(x, 2, put_chars_f, ff);
+		put_chrs(x, 7, put_chars_f, ff);
 		par_format.leftmargin += 2;
 		par_format.align = AL_LEFT;
 	} else {
@@ -979,7 +985,7 @@ void html_li(unsigned char *a)
 			}
 		} else sprintf(n, "%d", par_format.list_number);
 		put_chrs(n, strlen(n), put_chars_f, ff);
-		put_chrs(". ", 2, put_chars_f, ff);
+		put_chrs(".&nbsp;", 7, put_chars_f, ff);
 		par_format.leftmargin += strlen(n) + c + 2;
 		par_format.align = AL_LEFT;
 		par_format.list_number = 0;
@@ -1023,14 +1029,18 @@ void html_dd(unsigned char *a)
 
 void get_html_form(unsigned char *a, struct form *form)
 {
-	char *al;
-	if ((al = get_attr_val(a, "action"))) {
+	unsigned char *al;
+	unsigned char *ch;
+	if ((al = get_url_val(a, "action"))) {
 		char *all = al;
 		while (all[0] == ' ') all++;
 		while (all[0] && all[strlen(all) - 1] == ' ') all[strlen(all) - 1] = 0;
 		form->action = join_urls(format.href_base, all);
 		mem_free(al);
-	} else form->action = stracpy(format.href_base);
+	} else {
+		if ((ch = strchr(form->action = stracpy(format.href_base), POST_CHAR))) *ch = 0;
+	}
+	if (ch = strchr(form->action, '?')) *ch = 0;
 	if ((al = get_target(a))) {
 		form->target = al;
 	} else {
