@@ -743,6 +743,10 @@ struct part *format_html_part(unsigned char *start, unsigned char *end, int alig
 	int lm = margin;
 	int ef = empty_format;
 	struct form_control *fc;
+	if (ys < 0) {
+		internal("format_html_part: ys == %d", ys);
+		return NULL;
+	}
 	if (data) {
 		struct node *n;
 		if ((n = mem_alloc(sizeof(struct node)))) {
@@ -1128,6 +1132,7 @@ struct f_data_c *find_fd(struct session *ses, unsigned char *name, int depth, in
 	fd->name = stracpy(name);
 	fd->depth = depth;
 	fd->xp = x, fd->yp = y;
+	fd->search_word = &ses->search_word;
 	/*add_to_list(ses->scrn_frames, fd);*/
 	add_frame_to_list(ses, fd);
 	return fd;
@@ -1203,6 +1208,7 @@ void html_interpret(struct session *ses)
 	if (!ses->screen) {
 		if (!(ses->screen = mem_alloc(sizeof(struct f_data_c)))) return;
 		memset(ses->screen, 0, sizeof(struct f_data_c));
+		ses->screen->search_word = &ses->search_word;
 	}
 	if (!list_empty(ses->history)) l = &cur_loc(ses)->vs;
 	else l = NULL;
