@@ -135,8 +135,8 @@ unsigned char frame_table[81] = {
 	0xbb, 0xbb, 0xb9,	0xbb, 0xbb, 0xb9,	0xcb, 0xcb, 0xce,
 };
 
-unsigned char hline_table[3] = { 0x00, 0xc4, 0xcd };
-unsigned char vline_table[3] = { 0x00, 0xb3, 0xba };
+unsigned char hline_table[3] = { 0x20, 0xc4, 0xcd };
+unsigned char vline_table[3] = { 0x20, 0xb3, 0xba };
 
 struct table *new_table()
 {
@@ -200,7 +200,7 @@ void expand_cells(struct table *t, int x, int y)
 		if (t->y) {
 			for (i = 0; i < t->x; i++) if (CELL(t, i, t->y - 1)->rowspan == -1) {
 				for (j = t->y; j <= y; j++) {
-					CELL(t, j, i)->used = 1;
+					CELL(t, i, j)->used = 1;
 					CELL(t, i, j)->spanned = 1;
 					CELL(t, i, j)->rowspan = -1;
 					CELL(t, i, j)->colspan = CELL(t, i, t->y - 1)->colspan;
@@ -435,6 +435,8 @@ struct table *parse_table(unsigned char *html, unsigned char *eof, unsigned char
 		x++;
 		goto nc;
 	}
+	cell->mx = x;
+	cell->my = y;
 	cell->used = 1;
 	cell->start = en;
 	p = 1;
@@ -481,9 +483,11 @@ struct table *parse_table(unsigned char *html, unsigned char *eof, unsigned char
 			struct table_cell *sc;
 			if (!(sc = new_cell(t, x + k, y + j)) || sc->used) {
 				int l, m;
+				if (sc->mx == x && sc->my == y) continue;
+				/*internal("boo");*/
 				for (l = 0; l < k; l++) memset(CELL(t, x + l, y + j), 0, sizeof(struct table_cell));
 				rsp = j;
-				for (l = 0; l < i; l++) for (m = 0; m < j; m++) CELL(t, x + l, y + m)->rowspan = k;
+				for (l = 0; l < i; l++) for (m = 0; m < j; m++) CELL(t, x + l, y + m)->rowspan = j;
 				goto brk;
 			}
 			sc->used = sc->spanned = 1;
@@ -1003,10 +1007,13 @@ void display_table_frames(struct table *t, int x, int y)
 #define V_LINE(xx, yy) (V_LINE_X((xx), (yy)) == -1 ? 0 : V_LINE_X((xx), (yy)))
 	for (j = 0; j < t->y; j++) for (i = 0; i < t->x; i++) {
 		int x, y;
+		int xsp, ysp;
 		struct table_cell *cell = CELL(t, i, j);
 		if (!cell->used || cell->spanned) continue;
-		if (t->rules != R_NONE && t->rules != R_COLS) for (x = 0; x < cell->colspan; x++) {H_LINE_X(i + x, j) = t->cellsp; H_LINE_X(i + x, j + cell->rowspan) = t->cellsp;}
-		if (t->rules != R_NONE && t->rules != R_ROWS) for (y = 0; y < cell->rowspan; y++) {V_LINE_X(i, j + y) = t->cellsp; V_LINE_X(i + cell->colspan, j + y) = t->cellsp;}
+		if ((xsp = cell->colspan) == 0) xsp = t->x - i;
+		if ((ysp = cell->rowspan) == 0) ysp = t->y - j;
+		if (t->rules != R_NONE && t->rules != R_COLS) for (x = 0; x < xsp; x++) {H_LINE_X(i + x, j) = t->cellsp; H_LINE_X(i + x, j + ysp) = t->cellsp;}
+		if (t->rules != R_NONE && t->rules != R_ROWS) for (y = 0; y < ysp; y++) {V_LINE_X(i, j + y) = t->cellsp; V_LINE_X(i + xsp, j + y) = t->cellsp;}
 	}
 	if (t->rules == R_GROUPS) {
 		for (i = 1; i < t->x; i++) {
@@ -1221,6 +1228,7 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	}*/
 	if (p->cy + t->rh != cye) internal("size does not match; 1:%d, 2:%d", p->cy + t->rh, cye);
 	p->cy = cye;
+	p->cx = -1;
 
 	ret2:
 	p->link_num = t->link_num;
