@@ -58,6 +58,49 @@ unsigned char *parse_header_param(unsigned char *x, unsigned char *e)
 	return memacpy(y, lp);
 }
 
+/*
+ Parse string param="value", return value as new string
+ or NULL if any error.
+*/
+unsigned char *get_param(unsigned char *e, unsigned char *name)
+{
+	unsigned char *n, *start;
+	int i = 0;
+again:	
+	while (*e && upcase(*e++) != upcase(*name));
+	if (!*e) return NULL;
+	n = name + 1;
+	while (*n && upcase(*e) == upcase(*n)) e++, n++;
+	if (*n) goto again;
+	while (WHITECHAR(*e)) e++;
+	if (*e++ != '=') return NULL;
+	while (WHITECHAR(*e)) e++;
+
+	start = e;
+	if (!U(*e)) while (*e && !WHITECHAR(*e)) e++;
+	else {
+		char uu = *e++;
+		start++;
+		while (*e != uu) {
+			if (!*e) return NULL;
+			e++;
+		}
+	}
+	
+	while (start < e && *start == ' ') start++;
+	while (start < e && *(e - 1) == ' ') e--;
+	if (start == e) return NULL;
+	
+	if (!(n = mem_alloc(e - start + 1))) return NULL;
+	while (start < e) {
+		if (*start < ' ') n[i] = '.';
+		else n[i] = *start;
+		i++; start++;
+	}
+	n[i] = 0;
+	return n;
+}
+
 static int get_http_code(unsigned char *head, int *code, int *version)
 {
 	while (head[0] == ' ') head++;
@@ -103,7 +146,11 @@ void http_end_request(struct connection *c)
 			c->cache->incomplete = 0;
 		}
 	}
-	if (c->info && !((struct http_connection_info *)c->info)->close && (!http_bugs.bug_post_no_keepalive || !strchr(c->url, POST_CHAR))) {
+	if (c->info && !((struct http_connection_info *)c->info)->close 
+#ifdef HAVE_SSL
+	&& (!c->ssl) /* We won't keep alive ssl connections */
+#endif
+	&& (!http_bugs.bug_post_no_keepalive || !strchr(c->url, POST_CHAR))) {
 		add_keepalive_socket(c, HTTP_KEEPALIVE_TIMEOUT);
 	} else abort_connection(c);
 }
@@ -183,6 +230,11 @@ void http_send_header(struct connection *c)
 		add_to_str(&hdr, &l, "Host: ");
 		add_to_str(&hdr, &l, h);
 		mem_free(h);
+		if ((h = get_port_str(host))) {
+			add_to_str(&hdr, &l, ":");
+			add_to_str(&hdr, &l, h);
+			mem_free(h);
+		}
 		add_to_str(&hdr, &l, "\r\n");
 	}
 	add_to_str(&hdr, &l, "User-Agent: Links (" VERSION_STRING "; ");
@@ -225,6 +277,12 @@ void http_send_header(struct connection *c)
 		add_num_to_str(&hdr, &l, c->from);
 		add_to_str(&hdr, &l, "-\r\n");
 	}
+	if ((h = find_auth(host))) {
+		add_to_str(&hdr, &l, "Authorization: Basic ");
+		add_to_str(&hdr, &l, h);
+		add_to_str(&hdr, &l, "\r\n");
+		mem_free(h);
+	}
 	if (post) {
 		unsigned char *pd = strchr(post, '\n');
 		if (pd) {
@@ -458,6 +516,18 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 	}
 	if (e->head) mem_free(e->head);
 	e->head = head;
+#ifdef HAVE_SSL
+	if (c->ssl) {
+		int l = 0;
+		if (e->ssl_info) mem_free(e->ssl_info);
+		e->ssl_info = init_str();
+		add_num_to_str(&e->ssl_info, &l, SSL_get_cipher_bits(c->ssl, NULL));
+		add_to_str(&e->ssl_info, &l, "-bit ");
+		add_to_str(&e->ssl_info, &l, SSL_get_cipher_version(c->ssl));
+		add_to_str(&e->ssl_info, &l, " ");
+		add_to_str(&e->ssl_info, &l, (unsigned  char *)SSL_get_cipher_name(c->ssl));
+	}
+#endif
 	if (h == 204) {
 		setcstate(c, S_OK);
 		http_end_request(c);
@@ -470,6 +540,14 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 			e->redirect_get = h == 303;
 		}
 	}
+ 	/*if (h == 401) if ((d = parse_http_header(e->head, "WWW-Authenticate", NULL))) {
+ 		if (!strncasecmp(d, "Basic", 5)) {
+ 			add_auth_entry(host, get_param(d, "realm"));
+ 			add_questions_entry(do_auth_dialog);
+ 			setcstate(c, S_QUESTIONS);
+  		}
+ 		mem_free(d);
+  	}*/
 	kill_buffer_data(rb, a);
 	c->cache = e;
 	info->close = 0;
@@ -494,7 +572,7 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		unsigned char *ep;
 		int l = strtol(d, (char **)&ep, 10);
 		if (!*ep && l >= 0) {
-			info->length = l;
+			if (!info->close) info->length = l;
 			c->est_length = c->from + l;
 		}
 		mem_free(d);
@@ -508,6 +586,7 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		}
 		mem_free(d);
 	}
+	if (!info->close && info->length == -1) info->close = 1;
 	if ((d = parse_http_header(e->head, "Last-Modified", NULL))) {
 		if (e->last_modified && strcasecmp(e->last_modified, d)) {
 			delete_entry_content(e);
