@@ -75,6 +75,10 @@
 
 #include <termios.h>
 
+#ifdef HAVE_SSL
+#include <openssl/ssl.h>
+#endif
+
 #include "os_depx.h"
 
 #include "setup.h"
@@ -574,6 +578,9 @@ struct cache_entry {
 	struct list_head frag;
 	tcount count;
 	int refcount;
+#ifdef HAVE_SSL
+	unsigned char *ssl_info;
+#endif
 };
 
 struct fragment {
@@ -645,6 +652,10 @@ struct connection {
 	struct remaining_info prg;
 	int timer;
 	int detached;
+#ifdef HAVE_SSL
+	SSL *ssl;
+	int no_tsl;
+#endif
 };
 
 static inline int getpri(struct connection *c)
@@ -668,6 +679,7 @@ static inline int getpri(struct connection *c)
 #define S_GETH		4
 #define S_PROC		5
 #define S_TRANS		6
+#define S_QUESTIONS	7
 
 #define S_WAIT_REDIR		-999
 #define S_OK			-1000
@@ -698,6 +710,9 @@ static inline int getpri(struct connection *c)
 #define S_FTP_NO_FILE		-1304
 #define S_FTP_FILE_ERROR	-1305
 
+#define S_SSL_ERROR		-1400
+#define S_NO_SSL		-1401
+
 extern struct s_msg_dsc {
 	int n;
 	unsigned char *msg;
@@ -716,6 +731,17 @@ struct status {
 	struct remaining_info *prg;
 };
 
+struct http_auth_basic {
+	struct http_auth_basic *next;
+	struct http_auth_basic *prev;
+	int valid;
+	unsigned char *url;
+	int url_len;
+	unsigned char *realm;
+	unsigned char *uid;
+	unsigned char *passwd;
+};
+
 void check_queue();
 long connect_info(int);
 void send_connection_info(struct connection *c);
@@ -738,6 +764,12 @@ void add_blacklist_entry(unsigned char *, int);
 void del_blacklist_entry(unsigned char *, int);
 int get_blacklist_flags(unsigned char *);
 void free_blacklist();
+void update_noproxy();
+void free_noproxy();
+unsigned char *find_auth(unsigned char *);
+void add_auth_entry(unsigned char*, unsigned char *);
+void del_auth_entry(struct http_auth_basic *);
+void free_auth();
 
 #define BL_HTTP10	1
 #define BL_NO_CHARSET	2
@@ -758,6 +790,7 @@ unsigned char *get_host_name(unsigned char *);
 unsigned char *get_host_and_pass(unsigned char *);
 unsigned char *get_user_name(unsigned char *);
 unsigned char *get_pass(unsigned char *);
+unsigned char *get_port_str(unsigned char *);
 int get_port(unsigned char *);
 void (*get_protocol_handle(unsigned char *))(struct connection *);
 void (*get_external_protocol_function(unsigned char *))(struct session *, unsigned char *);
@@ -800,6 +833,15 @@ unsigned char *parse_header_param(unsigned char *, unsigned char *);
 void http_func(struct connection *);
 void proxy_func(struct connection *);
 
+
+/* https.c */
+
+void https_func(struct connection *c);
+#ifdef HAVE_SSL
+void ssl_finish(void);
+SSL *getSSL(void);
+#endif
+
 /* file.c */
 
 void file_func(struct connection *);
@@ -1013,6 +1055,7 @@ extern int current_language;
 void init_trans();
 void shutdown_trans();
 unsigned char *get_text_translation(unsigned char *, struct terminal *term);
+unsigned char *get_english_translation(unsigned char *);
 void set_language(int);
 int n_languages();
 unsigned char *language_name(int);
@@ -1314,7 +1357,7 @@ struct f_data_c {
 	int xl, yl; /* last pos of window */
 	struct link_bg *link_bg;
 	int link_bg_n;
-	unsigned char *search_word;
+	unsigned char **search_word;
 	struct view_state *vs;
 	int depth;
 };
@@ -1466,6 +1509,8 @@ unsigned char *get_current_title(struct session *, unsigned char *, size_t);
 
 unsigned char *get_current_link_url(struct session *, unsigned char *, size_t);
 
+void add_questions_entry(void *);
+
 /* bfu.c */
 
 struct memory_list {
@@ -1527,8 +1572,9 @@ struct history {
 #define D_END		0
 #define D_CHECKBOX	1
 #define D_FIELD		2
-#define D_BUTTON	3
-#define D_BOX		4
+#define D_FIELD_PASS	3
+#define D_BUTTON	4
+#define D_BOX		5
 
 #define B_ENTER		1
 #define B_ESC		2
@@ -1631,7 +1677,7 @@ void query_file(struct session *, unsigned char *, void (*)(struct session *, un
 void search_dlg(struct session *, struct f_data_c *, int);
 void search_back_dlg(struct session *, struct f_data_c *, int);
 void exit_prog(struct terminal *, void *, struct session *);
-
+void do_auth_dialog(struct session *);
 /* charsets.c */
 
 #include "codepage.h"
@@ -1882,8 +1928,20 @@ void format_table(unsigned char *, unsigned char *, unsigned char *, unsigned ch
 
 #define MAX_STR_LEN	1024
 
+struct option {
+	int p;
+	unsigned char *(*rd_cmd)(struct option *, unsigned char ***, int *);
+	unsigned char *(*rd_cfg)(struct option *, unsigned char *);
+	void (*wr_cfg)(struct option *, unsigned char **, int *);
+	int min, max;
+	void *ptr;
+	unsigned char *cfg_name;
+	unsigned char *cmd_name;
+};
+
 unsigned char *parse_options(int, unsigned char *[]);
 void init_home();
+unsigned char *get_token(unsigned char **line);
 void load_config();
 void write_config(struct terminal *);
 void write_html_config(struct terminal *);
@@ -1928,6 +1986,7 @@ extern struct rgb default_vlink;
 
 extern unsigned char http_proxy[];
 extern unsigned char ftp_proxy[];
+extern unsigned char no_proxy_for[];
 extern unsigned char download_dir[];
 
 struct http_bugs {
@@ -2022,3 +2081,61 @@ struct bookmark *create_bookmark(const unsigned char *, const unsigned char *);
 /* Launches add dialogs */
 void launch_bm_add_link_dialog(struct terminal *,struct dialog_data *,struct session *);
 void launch_bm_add_doc_dialog(struct terminal *,struct dialog_data *,struct session *);
+
+/* kbdbind.c */
+
+#define KM_MAIN		0
+#define KM_EDIT		1
+#define KM_MAX		2
+
+enum {
+	ACT_ADD_BOOKMARK,
+	ACT_AUTO_COMPLETE,
+	ACT_BACK,
+	ACT_BACKSPACE,
+	ACT_BOOKMARK_MANAGER,
+	ACT_COPY_CLIPBOARD,
+	ACT_CUT_CLIPBOARD,
+	ACT_DELETE,
+	ACT_DOCUMENT_INFO,
+	ACT_DOWN,
+	ACT_DOWNLOAD,
+	ACT_END,
+	ACT_ENTER,
+	ACT_FILE_MENU,
+	ACT_FIND_NEXT,
+	ACT_FIND_NEXT_BACK,
+	ACT_GOTO_URL,
+	ACT_GOTO_URL_CURRENT,
+	ACT_HEADER_INFO,
+	ACT_HOME,
+	ACT_KILL_TO_BOL,
+	ACT_KILL_TO_EOL,
+	ACT_LEFT,
+	ACT_MENU,
+	ACT_NEXT_FRAME,
+	ACT_PAGE_DOWN,
+	ACT_PAGE_UP,
+	ACT_PASTE_CLIPBOARD,
+	ACT_PREVIOUS_FRAME,
+	ACT_QUIT,
+	ACT_RELOAD,
+	ACT_RIGHT,
+	ACT_SCROLL_DOWN,
+	ACT_SCROLL_LEFT,
+	ACT_SCROLL_RIGHT,
+	ACT_SCROLL_UP,
+	ACT_SEARCH,
+	ACT_SEARCH_BACK,
+	ACT_TOGGLE_DISPLAY_IMAGES,
+	ACT_TOGGLE_HTML_PLAIN,
+	ACT_UP,
+	ACT_VIEW_IMAGE,
+	ACT_ZOOM_FRAME
+};
+
+void init_keymaps();
+void free_keymaps();
+int kbd_action(int, struct event *);
+unsigned char *bind_rd(struct option *, unsigned char *);
+unsigned char *unbind_rd(struct option *, unsigned char *);
