@@ -186,7 +186,7 @@ void end_dump(struct status *stat, void *p)
 				detach_connection(stat, dump_pos);
 			}
 		}
-		if (stat->state > 0) return;
+		if (stat->state >= 0) return;
 	} else if (ce) {
 		struct document_options o;
 		struct view_state *vs;
@@ -220,8 +220,7 @@ void end_dump(struct status *stat, void *p)
 	}
 	if (stat->state != S_OK) {
 		unsigned char *m = get_err_msg(stat->state);
-		fprintf(stderr, "%s\n", m);
-		mem_free(m);
+		fprintf(stderr, "%s\n", get_english_translation(m));
 		retval = RET_ERROR;
 		goto terminate;
 	}
@@ -245,6 +244,7 @@ void init()
 	init_trans();
 	set_sigcld();
 	init_home();
+	init_keymaps();
 	if (!(u = parse_options(ac - 1, av + 1))) goto ttt;
 	if (!no_connect && (uh = bind_to_af_unix()) != -1) {
 		if (!(info = create_session_info(base_session, u, &len))) goto ttt;
@@ -297,12 +297,18 @@ void terminate_all_subsystems()
 	check_bottom_halves();
 	free_all_itrms();
 	abort_all_connections();
+#ifdef HAVE_SSL
+	ssl_finish();
+#endif
 	shrink_memory(1);
 	if (init_b) save_url_history();
 	free_history_lists();
 	free_term_specs();
 	free_types();
+	free_noproxy();
+	free_auth();
 	if (init_b) finalize_bookmarks();
+	free_keymaps();
 	free_conv_table();
 	free_blacklist();
 	if (init_b) cleanup_cookies();
