@@ -205,13 +205,14 @@ void free_connection_data(struct connection *c)
 
 void send_connection_info(struct connection *c)
 {
+	int st = c->state;
 	tcount count = c->count;
 	struct status *stat = c->statuss.next;
 	while ((void *)stat != &c->statuss) {
 		stat->ce = c->cache;
 		stat = stat->next;
 		if (stat->prev->end) stat->prev->end(stat->prev, stat->prev->data);
-		if (connection_disappeared(c, count)) return;
+		if (st >= 0 && connection_disappeared(c, count)) return;
 	}
 }
 
@@ -322,6 +323,13 @@ void sort_queue()
 
 void interrupt_connection(struct connection *c)
 {
+#ifdef HAVE_SSL
+	if (c->ssl == (void *)-1) c->ssl = 0;
+	if(c->ssl) {
+		SSL_free(c->ssl);
+		c->ssl=NULL;
+	}
+#endif
 	close_socket(&c->sock1);
 	free_connection_data(c);
 }
@@ -505,13 +513,15 @@ void check_queue()
 #endif
 }
 
+int check_noproxy(unsigned char *url);
+
 unsigned char *get_proxy(unsigned char *url)
 {
 	int l = strlen(url);
 	unsigned char *proxy = NULL;
 	unsigned char *u;
-	if (*http_proxy && l >= 7 && !casecmp(url, "http://", 7)) proxy = http_proxy;
-	if (*ftp_proxy && l >= 6 && !casecmp(url, "ftp://", 6)) proxy = ftp_proxy;
+	if (*http_proxy && l >= 7 && !casecmp(url, "http://", 7) && !check_noproxy(url)) proxy = http_proxy;
+	if (*ftp_proxy && l >= 6 && !casecmp(url, "ftp://", 6) && !check_noproxy(url)) proxy = ftp_proxy;
 	if (!(u = mem_alloc(l + 1 + (proxy ? strlen(proxy) + 9 : 0)))) return NULL;
 	if (proxy) strcpy(u, "proxy://"), strcat(u, proxy), strcat(u, "/");
 	else *u = 0;
@@ -549,7 +559,6 @@ int load_url(unsigned char *url, struct status *stat, int pri, int no_cache)
 	}
 	if (!(u = get_proxy(url))) {
 		if (stat) stat->end(stat, stat->data);
-		mem_free(c);
 		return -1;
 	}
 	foreach(c, queue) if (!c->detached && !strcmp(c->url, u)) {
@@ -785,6 +794,174 @@ void free_blacklist()
 	free_list(blacklist);
 }
 
+struct noproxy {
+	unsigned char *st;
+	int len;
+} *noproxy_list = DUMMY;
+
+int check_noproxy(unsigned char *url)
+{
+	unsigned char *host;
+	struct noproxy *nl = noproxy_list;
+	if (nl == DUMMY) return 0;
+	if (!(host = get_host_name(url))) return 0;
+	while (nl->st) {
+		int l = strlen(host);
+		if ((l == nl->len) || (l > nl->len && host[l - nl->len - 1] == '.')) if (!strncasecmp(host + l - nl->len, nl->st, nl->len)) {
+			mem_free(host);
+			return 1;
+		}
+		nl++;
+	}
+	mem_free(host);
+	return 0;
+}
+
+void update_noproxy()
+{
+	int n = 0;
+	unsigned char *list = no_proxy_for;
+	struct noproxy *nl;
+	if (!*list) return;
+	while  (*list) {
+		if (*list == ',') n++;
+		list++;
+	}
+	if (!(nl = mem_realloc(noproxy_list, (n + 2) * sizeof(struct noproxy)))) return;
+	noproxy_list = nl;
+	list = no_proxy_for;
+	while(*list == ' '|| *list == ',') list++;
+	if (!*list) goto end;
+	nl->st = list;
+	while (*list) {
+		if (*list == ',') {
+			nl->len = list - nl->st;
+			while (*(nl->st + nl->len - 1) == ' ') {
+				if (!--nl->len) {
+					nl--;
+					break;
+				}
+			}
+			nl++;
+			list++;
+			while(*list == ' ' || *list == ',') list++;
+			if (!*list) goto end;
+			nl->st = list;
+		}
+		list++;
+	}
+	(nl++)->len = list - nl->st;
+	end:	
+	nl->st = 0;
+	nl->len  = 0;
+}
+
+void free_noproxy()
+{
+	mem_free(noproxy_list);
+}
+
+struct list_head http_auth_basic_list = { &http_auth_basic_list, &http_auth_basic_list };
+
+struct http_auth_basic *find_auth_entry(unsigned char *url)
+{
+	struct http_auth_basic *a;
+	foreach(a, http_auth_basic_list) if (!strncmp(a->url, url, a->url_len)) return a;
+	return NULL;
+}
+
+void add_auth_entry(unsigned char* url, unsigned char *realm)
+{
+	struct http_auth_basic *a;
+	if ((a = find_auth_entry(url))) {
+		a->valid = 0;
+		if (a->realm) mem_free(a->realm);
+		a->realm = realm;
+		del_from_list(a);
+		add_to_list(http_auth_basic_list, a);
+		return;
+	}
+	if (!(a = mem_alloc(sizeof(struct http_auth_basic)))) return;
+	memset(a, 0, sizeof(struct http_auth_basic));
+	while (url[a->url_len] && !end_of_dir(url[a->url_len])) a->url_len++;
+	while (a->url_len && url[a->url_len - 1] != '/') a->url_len--;
+	if (a->url_len < 8) goto err;
+	if (!(a->url = mem_alloc(a->url_len + 1))) {
+		err:
+		mem_free(a);
+		return;
+	}
+	memcpy(a->url, url, a->url_len);
+	a->url[a->url_len] = 0;
+	a->realm = realm;
+	add_to_list(http_auth_basic_list, a);
+	return;
+}
+
+unsigned char *base64_encode(unsigned char *);
+
+unsigned char *find_auth(unsigned char *url)
+{
+	struct http_auth_basic *a = NULL;
+	unsigned char *uid, *b64;
+	if ((a = find_auth_entry(url))) {
+		int n;
+		if (!(uid = mem_alloc(strlen(a->uid) + strlen(a->passwd) + 2))) return NULL;
+		strcpy(uid, a->uid);
+		uid[n = strlen(a->uid)] = ':';
+		strcpy(&uid[n + 1], a->passwd);
+		b64 = base64_encode(uid);
+		mem_free(uid);
+		return b64;
+	}
+	return NULL;
+}
+
+void del_auth_entry(struct http_auth_basic *a)
+{
+		if (a->url) mem_free(a->url);
+		if (a->realm) mem_free(a->realm);
+		if (a->uid) mem_free(a->uid);
+		if (a->passwd) mem_free(a->passwd);
+		del_from_list(a);
+		mem_free(a);
+}
+
+void free_auth()
+{
+	while (!list_empty(http_auth_basic_list)) del_auth_entry(http_auth_basic_list.next);
+}
+
+unsigned char base64_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+unsigned char *base64_encode(unsigned char *in)
+{
+	unsigned char *out, *outstr;
+	int inlen = strlen(in);
+	if (!(outstr = out = mem_alloc(((inlen / 3) + 1) * 4 + 1 ))) return NULL;
+	while (inlen >= 3) {
+		*out++ = base64_chars[ (int)(*in >> 2) ];
+		*out++ = base64_chars[ (int)((*in << 4 | *(in + 1) >> 4) & 63) ];
+		*out++ = base64_chars[ (int)((*(in + 1) << 2 | *(in + 2) >> 6) & 63) ];
+		*out++ = base64_chars[ (int)(*(in + 2) & 63) ];
+		inlen -= 3; in += 3;
+	}
+	if (inlen == 1) {
+		*out++ = base64_chars[ (int)(*in >> 2) ];
+		*out++ = base64_chars[ (int)(*in << 4 & 63) ];
+		*out++ = '=';
+		*out++ = '=';
+	}
+	if (inlen == 2) {
+		*out++ = base64_chars[ (int)(*in >> 2) ];
+		*out++ = base64_chars[ (int)((*in << 4 | *(in + 1) >> 4) & 63) ];
+		*out++ = base64_chars[ (int)((*(in + 1) << 2) & 63) ];
+		*out++ = '=';
+	}
+	*out = 0;
+	return outstr;
+}
+
 struct s_msg_dsc msg_dsc[] = {
 	S_WAIT,			TEXT(T_WAITING_IN_QUEUE),
 	S_DNS,			TEXT(T_LOOKING_UP_HOST),
@@ -822,6 +999,10 @@ struct s_msg_dsc msg_dsc[] = {
 	S_FTP_PORT,		TEXT(T_FTP_PORT_COMMAND_FAILED),
 	S_FTP_NO_FILE,		TEXT(T_FILE_NOT_FOUND),
 	S_FTP_FILE_ERROR,	TEXT(T_FTP_FILE_ERROR),
-
+#ifdef HAVE_SSL
+	S_SSL_ERROR,		TEXT(T_SSL_ERROR),
+#else
+	S_NO_SSL,		TEXT(T_NO_SSL),
+#endif
 	0,		NULL
 };
