@@ -83,7 +83,7 @@ unsigned char *get_stat_msg(struct status *stat, struct terminal *term)
 		add_to_str(&m, &l, ", ");
 		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME)
 			add_to_str(&m, &l, _(TEXT(T_AVG), term)), add_to_str(&m, &l, " ");
-		add_xnum_to_str(&m, &l, stat->prg->loaded * 10 / (stat->prg->elapsed / 100));
+		add_xnum_to_str(&m, &l, (longlong)stat->prg->loaded * 10 / (stat->prg->elapsed / 100));
 		add_to_str(&m, &l, "/s");
 		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME) 
 			add_to_str(&m, &l, ", "), add_to_str(&m, &l, _(TEXT(T_CUR), term)), add_to_str(&m, &l, " "),
@@ -368,7 +368,7 @@ void download_window_function(struct dialog_data *dlg)
 			add_to_str(&m, &l, _(TEXT(T_AVERAGE_SPEED), term));
 		else add_to_str(&m, &l, _(TEXT(T_SPEED), term));
 		add_to_str(&m, &l, " ");
-		add_xnum_to_str(&m, &l, stat->prg->loaded * 10 / (stat->prg->elapsed / 100));
+		add_xnum_to_str(&m, &l, (longlong)stat->prg->loaded * 10 / (stat->prg->elapsed / 100));
 		add_to_str(&m, &l, "/s");
 		if (stat->prg->elapsed >= CURRENT_SPD_AFTER * SPD_DISP_TIME) 
 			add_to_str(&m, &l, ", "), add_to_str(&m, &l, _(TEXT(T_CURRENT_SPEED), term)), add_to_str(&m, &l, " "),
@@ -383,7 +383,7 @@ void download_window_function(struct dialog_data *dlg)
 			add_to_str(&m, &l, _(TEXT(T_ESTIMATED_TIME), term));
 			add_to_str(&m, &l, " ");
 			/*add_time_to_str(&m, &l, stat->prg->elapsed / 1000 * stat->prg->size / stat->prg->loaded * 1000 - stat->prg->elapsed);*/
-			add_time_to_str(&m, &l, (stat->prg->size - stat->prg->pos) / (stat->prg->loaded * 10 / (stat->prg->elapsed / 100)) * 1000);
+			add_time_to_str(&m, &l, (stat->prg->size - stat->prg->pos) / ((longlong)stat->prg->loaded * 10 / (stat->prg->elapsed / 100)) * 1000);
 		}
 	} else m = stracpy(_(get_err_msg(stat->state), term));
 	if (!m) return;
@@ -733,6 +733,7 @@ void tp_free(struct session *ses)
 	mem_free(ses->tq_url);
 	ses->tq_url = NULL;
 	if (ses->tq_goto_position) mem_free(ses->tq_goto_position), ses->tq_goto_position = NULL;
+	ses->tq_ce = NULL;
 }
 
 void tp_cancel(struct session *ses)
@@ -1022,6 +1023,32 @@ void display_timer(struct session *ses)
 	process_file_requests(ses);
 }
 	
+struct list_head questions_queue = {&questions_queue, &questions_queue};
+
+struct questions_entry {
+	struct questions_entry *next;
+	struct questions_entry *prev;
+	void (*callback)(struct session *);
+};
+
+void check_questions_queue(struct session *ses)
+{
+	struct questions_entry *q;
+	while (!list_empty(questions_queue)) {
+		q = questions_queue.next;
+		q->callback(ses);
+		del_from_list(q);
+		mem_free(q);
+	}
+}
+
+void add_questions_entry(void *callback)
+{
+	struct questions_entry *q;
+	if (!(q = mem_alloc(sizeof(struct questions_entry)))) return;
+	q->callback = callback;
+	add_to_list(questions_queue, q);
+}
 
 void end_load(struct status *stat, struct session *ses)
 {
@@ -1045,6 +1072,7 @@ void end_load(struct status *stat, struct session *ses)
 	if (stat->state < 0 && stat->state != S_OK && d != 2) {
 		print_error_dialog(ses, stat, TEXT(T_ERROR));
 	}
+	if (stat->state == S_QUESTIONS) check_questions_queue(ses);
 	print_screen_status(ses);
 }
 
@@ -1058,6 +1086,7 @@ void doc_end_load(struct status *stat, struct session *ses)
 		process_file_requests(ses);
 		if (stat->state != S_OK) print_error_dialog(ses, stat, TEXT(T_ERROR));
 	} else if (ses->display_timer == -1) display_timer(ses);
+	if (stat->state == S_QUESTIONS) check_questions_queue(ses);
 	print_screen_status(ses);
 }
 
@@ -1304,6 +1333,7 @@ void destroy_session(struct session *ses)
 	if (ses->goto_position) mem_free(ses->goto_position);
 	if (ses->imgmap_href_base) mem_free(ses->imgmap_href_base);
 	if (ses->imgmap_target_base) mem_free(ses->imgmap_target_base);
+	if (ses->tq_ce) ses->tq_ce->refcount--;
 	if (ses->tq_url) {
 		change_connection(&ses->tq, NULL, PRI_CANCEL);
 		mem_free(ses->tq_url);
