@@ -443,17 +443,17 @@ void get_searched(struct f_data_c *scr, struct point **pt, int *pl)
 	unsigned char c;
 	struct point *points = DUMMY;
 	int len = 0;
-	if (!scr->search_word || !scr->search_word[0]) return;
+	if (!scr->search_word || !*scr->search_word || !(*scr->search_word)[0]) return;
 	get_search_data(scr->f_data);
-	l = strlen(scr->search_word);
-	c = scr->search_word[0];
+	l = strlen(*scr->search_word);
+	c = (*scr->search_word)[0];
 	if (get_range(scr->f_data, scr->vs->view_pos, scr->yw, l, &s1, &s2)) goto ret;
 	for (; s1 <= s2; s1++) {
 		int i, j;
 		if (srch_cmp(s1->c, c)) {
 			c:continue;
 		}
-		for (i = 1; i < l; i++) if (srch_cmp(s1[i].c, scr->search_word[i])) goto c;
+		for (i = 1; i < l; i++) if (srch_cmp(s1[i].c, (*scr->search_word)[i])) goto c;
 		for (i = 0; i < l; i++) for (j = 0; j < s1[i].n; j++) {
 			int x = s1[i].x + j + xp - vx;
 			int y = s1[i].y + yp - vy;
@@ -485,7 +485,7 @@ void draw_searched(struct terminal *t, struct f_data_c *scr)
 	int vy = scr->vs->view_pos;
 	struct point *pt;
 	int len, i;
-	if (!scr->search_word || !scr->search_word[0]) return;
+	if (!scr->search_word || !*scr->search_word || !(*scr->search_word)[0]) return;
 	get_searched(scr, &pt, &len);
 	for (i = 0; i < len; i++) {
 		int x = pt[i].x + xp - vx, y = pt[i].y + yp - vy;
@@ -730,7 +730,7 @@ void draw_doc(struct terminal *t, struct f_data_c *scr, int active)
 	}
 	vx = vs->view_posx;
 	vy = vs->view_pos;
-	if (scr->xl == vx && scr->yl == vy && scr->xl != -1 && !scr->search_word) {
+	if (scr->xl == vx && scr->yl == vy && scr->xl != -1 && (!scr->search_word || !*scr->search_word || !(*scr->search_word)[0])) {
 		clear_link(t, scr);
 		draw_forms(t, scr);
 		if (active) draw_current_link(t, scr);
@@ -751,7 +751,7 @@ void draw_doc(struct terminal *t, struct f_data_c *scr, int active)
 	}
 	draw_forms(t, scr);
 	if (active) draw_current_link(t, scr);
-	if (scr->search_word) scr->xl = scr->yl = -1;
+	if (scr->search_word && *scr->search_word && (*scr->search_word)[0]) scr->xl = scr->yl = -1;
 }
 
 void draw_frames(struct session *ses)
@@ -793,7 +793,6 @@ void draw_formatted(struct session *ses)
 	if (!ses->screen->vs && !list_empty(ses->history))
 		ses->screen->vs = &cur_loc(ses)->vs;
 	ses->screen->xl = ses->screen->yl = -1;
-	ses->screen->search_word = ses->search_word;
 	draw_doc(ses->term, ses->screen, 1);
 	draw_frames(ses);
 	print_screen_status(ses);
@@ -1410,7 +1409,6 @@ void selected_item(struct terminal *term, void *pitem, struct session *ses)
 	l = &f->f_data->links[f->vs->current_link];
 	if (l->type != L_SELECT) return;
 	if ((fs = find_form_state(f, l->form))) fs->state = item;
-	f->search_word = ses->search_word;
 	draw_doc(ses->term, f, 1);
 	print_screen_status(ses);
 	redraw_from_window(ses->win);
@@ -1445,127 +1443,146 @@ int field_op(struct session *ses, struct f_data_c *f, struct link *l, struct eve
 	if (!(fs = find_form_state(f, form))) return 0;
 	if (!fs->value) return 0;
 	if (ev->ev == EV_KBD) {
-		if (ev->x == KBD_LEFT) fs->state = fs->state ? fs->state - 1 : 0;
-		else if (ev->x == KBD_RIGHT) fs->state = fs->state < strlen(fs->value) ? fs->state + 1 : strlen(fs->value);
-		else if (ev->x == KBD_HOME) {
-			if (form->type == FC_TEXTAREA) {
-				struct line_info *ln;
-				if ((ln = format_text(fs->value, form->cols, form->wrap))) {
-					int y;
-					for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
-						fs->state = ln[y].st - fs->value;
-						goto x;
-					}
-					fs->state = 0;
-					x:
-					mem_free(ln);
-				}
-			} else fs->state = 0;
-		} else if (ev->x == KBD_UP) {
-			if (form->type == FC_TEXTAREA) {
-				struct line_info *ln;
-				if ((ln = format_text(fs->value, form->cols, form->wrap))) {
-					int y;
-					rep1:
-					for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
-						if (!y) {
-							mem_free(ln);
-							goto b;
+		switch (kbd_action(KM_EDIT, ev)) {
+			case ACT_LEFT: fs->state = fs->state ? fs->state - 1 : 0; break;
+			case ACT_RIGHT: fs->state = fs->state < strlen(fs->value) ? fs->state + 1 : strlen(fs->value); break;
+			case ACT_HOME:
+				if (form->type == FC_TEXTAREA) {
+					struct line_info *ln;
+					if ((ln = format_text(fs->value, form->cols, form->wrap))) {
+						int y;
+						for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
+							fs->state = ln[y].st - fs->value;
+							goto x;
 						}
-						fs->state -= ln[y].st - ln[y-1].st;
-						if (fs->value + fs->state > ln[y-1].en) fs->state = ln[y-1].en - fs->value;
-						goto xx;
+						fs->state = 0;
+						x:
+						mem_free(ln);
 					}
-					mem_free(ln);
-					goto b;
-					xx:
-					if (rep) goto rep1;
-					mem_free(ln);
-				}
-				
-			} else x = 0;
-		} else if (ev->x == KBD_DOWN) {
-			if (form->type == FC_TEXTAREA) {
-				struct line_info *ln;
-				if ((ln = format_text(fs->value, form->cols, form->wrap))) {
-					int y;
-					rep2:
-					for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
-						if (!ln[y+1].st) {
-							mem_free(ln);
-							goto b;
+				} else fs->state = 0;
+				break;
+			case ACT_UP:
+				if (form->type == FC_TEXTAREA) {
+					struct line_info *ln;
+					if ((ln = format_text(fs->value, form->cols, form->wrap))) {
+						int y;
+						rep1:
+						for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
+							if (!y) {
+								mem_free(ln);
+								goto b;
+							}
+							fs->state -= ln[y].st - ln[y-1].st;
+							if (fs->value + fs->state > ln[y-1].en) fs->state = ln[y-1].en - fs->value;
+							goto xx;
 						}
-						fs->state += ln[y+1].st - ln[y].st;
-						if (fs->value + fs->state > ln[y+1].en) fs->state = ln[y+1].en - fs->value;
-						goto yy;
+						mem_free(ln);
+						goto b;
+						xx:
+						if (rep) goto rep1;
+						mem_free(ln);
 					}
-					mem_free(ln);
-					goto b;
-					yy:
-					if (rep) goto rep2;
-					mem_free(ln);
-				}
-				
-			} else x = 0;
-		} else if (ev->x == KBD_END) {
-			if (form->type == FC_TEXTAREA) {
-				struct line_info *ln;
-				if ((ln = format_text(fs->value, form->cols, form->wrap))) {
-					int y;
-					for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
-						fs->state = ln[y].en - fs->value;
-						goto y;
+				} else x = 0;
+				break;
+			case ACT_DOWN:
+				if (form->type == FC_TEXTAREA) {
+					struct line_info *ln;
+					if ((ln = format_text(fs->value, form->cols, form->wrap))) {
+						int y;
+						rep2:
+						for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
+							if (!ln[y+1].st) {
+								mem_free(ln);
+								goto b;
+							}
+							fs->state += ln[y+1].st - ln[y].st;
+							if (fs->value + fs->state > ln[y+1].en) fs->state = ln[y+1].en - fs->value;
+							goto yy;
+						}
+						mem_free(ln);
+						goto b;
+						yy:
+						if (rep) goto rep2;
+						mem_free(ln);
+					}
+				} else x = 0;
+				break;
+			case ACT_END:
+				if (form->type == FC_TEXTAREA) {
+					struct line_info *ln;
+					if ((ln = format_text(fs->value, form->cols, form->wrap))) {
+						int y;
+						for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
+							fs->state = ln[y].en - fs->value;
+							goto y;
+						}
+						fs->state = strlen(fs->value);
+						y:
+						mem_free(ln);
+					}
+				} else fs->state = strlen(fs->value);
+				break;
+			case ACT_COPY_CLIPBOARD:
+				set_clipboard_text(fs->value);
+				break;
+			case ACT_CUT_CLIPBOARD:
+				set_clipboard_text(fs->value);
+				if (!form->ro) fs->value[0] = 0;
+				fs->state = 0;
+				break;
+			case ACT_PASTE_CLIPBOARD: {
+				char *clipboard = get_clipboard_text();
+				if (!form->ro && strlen(clipboard) <= form->maxlength) {
+					unsigned char *v;
+					if (v = mem_realloc(fs->value, strlen(clipboard) +1)) {
+						fs->value = v;
+						memmove(v , clipboard, strlen(clipboard) +1);
+						fs->state = strlen(fs->value);
 					}
-					fs->state = strlen(fs->value);
-					y:
-					mem_free(ln);
-				}
-			} else fs->state = strlen(fs->value);
-		} else if (!ev->y && (ev->x >= 32 && ev->x < 256)) {
-			if (!form->ro && strlen(fs->value) < form->maxlength) {
-				unsigned char *v;
-				if ((v = mem_realloc(fs->value, strlen(fs->value) + 2))) {
-					fs->value = v;
-					memmove(v + fs->state + 1, v + fs->state, strlen(v + fs->state) + 1);
-					v[fs->state++] = ev->x;
 				}
+				mem_free(clipboard);
+				break;
 			}
-		} else if (ev->x == KBD_INS && ev->y == KBD_CTRL || upcase(ev->x) == 'B' && ev->y == KBD_CTRL) {
-			set_clipboard_text(fs->value);
-		} else if (ev->x == KBD_DEL && ev->y == KBD_SHIFT || upcase(ev->x) == 'X' && ev->y == KBD_CTRL) {
-			set_clipboard_text(fs->value);
-			if (!form->ro) fs->value[0] = 0;
-			fs->state = 0;
-		} else if (ev->x == KBD_INS && ev->y == KBD_SHIFT || upcase(ev->x) == 'V' && ev->y == KBD_CTRL) {
-			char * clipboard = get_clipboard_text();
-			if (!form->ro && strlen(clipboard) <= form->maxlength) {
-				unsigned char *v;
-				if (v = mem_realloc(fs->value, strlen(clipboard) +1)) {
-					fs->value = v;
-					memmove(v , clipboard, strlen(clipboard) +1);
-					fs->state = strlen(fs->value);
+			case ACT_ENTER:
+				if (form->type == FC_TEXTAREA) {
+					if (!form->ro && strlen(fs->value) < form->maxlength) {
+						unsigned char *v;
+						if ((v = mem_realloc(fs->value, strlen(fs->value) + 2))) {
+							fs->value = v;
+							memmove(v + fs->state + 1, v + fs->state, strlen(v + fs->state) + 1);
+							v[fs->state++] = '\n';
+						}
+					}
 				}
-			}
-			mem_free(clipboard);
-		} else if (ev->x == KBD_ENTER && form->type == FC_TEXTAREA) {
-			if (!form->ro && strlen(fs->value) < form->maxlength) {
-				unsigned char *v;
-				if ((v = mem_realloc(fs->value, strlen(fs->value) + 2))) {
-					fs->value = v;
-					memmove(v + fs->state + 1, v + fs->state, strlen(v + fs->state) + 1);
-					v[fs->state++] = '\n';
+				else x = 0;
+				break;
+			case ACT_BACKSPACE:
+				if (!form->ro && fs->state) memmove(fs->value + fs->state - 1, fs->value + fs->state, strlen(fs->value + fs->state) + 1), fs->state--;
+				break;
+			case ACT_DELETE:
+				if (!form->ro && fs->state < strlen(fs->value)) memmove(fs->value + fs->state, fs->value + fs->state + 1, strlen(fs->value + fs->state));
+				break;
+			case ACT_KILL_TO_BOL:
+				if (!form->ro) memmove(fs->value, fs->value + fs->state, strlen(fs->value + fs->state) + 1);
+				fs->state = 0;
+				break;
+		    	case ACT_KILL_TO_EOL:
+				fs->value[fs->state] = 0;
+				break;
+			default:
+				if (!ev->y && (ev->x >= 32 && ev->x < 256)) {
+					if (!form->ro && strlen(fs->value) < form->maxlength) {
+						unsigned char *v;
+						if ((v = mem_realloc(fs->value, strlen(fs->value) + 2))) {
+							fs->value = v;
+							memmove(v + fs->state + 1, v + fs->state, strlen(v + fs->state) + 1);
+							v[fs->state++] = ev->x;
+						}
+					}
+				} else {
+					b:
+					x = 0;
 				}
-			}
-		} else if (ev->x == KBD_BS) {
-			if (!form->ro && fs->state) memmove(fs->value + fs->state - 1, fs->value + fs->state, strlen(fs->value + fs->state) + 1), fs->state--;
-		} else if (ev->x == KBD_DEL) {
-			if (!form->ro && fs->state < strlen(fs->value)) memmove(fs->value + fs->state, fs->value + fs->state + 1, strlen(fs->value + fs->state));
-		} else if (upcase(ev->x) == 'U' && ev->y == KBD_CTRL) {
-			if (!form->ro) memmove(fs->value, fs->value + fs->state, strlen(fs->value + fs->state) + 1);
-			fs->state = 0;
-		} else {
-			b:
-			x = 0;
 		}
 	} else x = 0;
 	if (x) {
@@ -1677,7 +1694,6 @@ void find_next(struct session *ses, struct f_data_c *f, int a)
 				if (min < f->vs->view_posx) f->vs->view_posx = min;
 			}
 			set_link(f);
-			f->search_word = ses->search_word;
 			find_next_link_in_search(f, ses->search_direction * 2);
 			/*draw_doc(ses->term, f, 1);
 			print_screen_status(ses);
@@ -1691,7 +1707,6 @@ void find_next(struct session *ses, struct f_data_c *f, int a)
 			p -= f->yw;
 		}
 	} while ((c += f->yw) < f->f_data->y + f->yw);
-	f->search_word = ses->search_word;
 	/*draw_doc(ses->term, f, 1);
 	print_screen_status(ses);
 	redraw_from_window(ses->win);*/
@@ -1740,11 +1755,12 @@ void goto_link_number(struct session *ses, unsigned char *num)
 	f->vs->current_link = n - 1;
 	link = &f->f_data->links[f->vs->current_link];
 	check_vs(f);
-	if (link->type != L_AREA && (link->type != L_FIELD || !has_form_submit(f->f_data, link->form))) enter(ses, f, 0);
+	if (link->type != L_AREA && link->type != L_FIELD) enter(ses, f, 0);
 }
 
 void frm_download(struct session *, struct f_data_c *);
-
+void send_image(struct terminal *term, void *xxx, struct session *ses);
+    
 int frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 {
 	int x = 1;
@@ -1756,69 +1772,69 @@ int frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 		return 1;
 	}
 	if (ev->ev == EV_KBD) {
-		if (ev->x == KBD_PAGE_DOWN || (ev->x == ' ' && (!ev->y || ev->y == KBD_CTRL))) rep_ev(ses, fd, page_down, 0);
-		else if (ev->x == KBD_PAGE_UP || (upcase(ev->x) == 'B' && (!ev->y || ev->y == KBD_CTRL))) rep_ev(ses, fd, page_up, 0);
-		else if (ev->x == KBD_DOWN) rep_ev(ses, fd, down, 0);
-		else if (ev->x == KBD_UP) rep_ev(ses, fd, up, 0);
-		/* Copy current link to clipboard */
-		else if (ev->x == KBD_INS && ev->y == KBD_CTRL || upcase(ev->x) == 'C' && ev->y == KBD_CTRL) {
-			char *current_link = print_current_link(ses);
-			set_clipboard_text( current_link );
-			mem_free(current_link);
-		}
-		else if (ev->x == KBD_INS || (upcase(ev->x) == 'P' && ev->y == KBD_CTRL)) rep_ev(ses, fd, scroll, -1 - !ses->kbdprefix.rep);
-		else if (ev->x == KBD_DEL || (upcase(ev->x) == 'N' && ev->y == KBD_CTRL)) rep_ev(ses, fd, scroll, 1 + !ses->kbdprefix.rep);
-		else if (ev->x == '[') rep_ev(ses, fd, hscroll, -1 - 7 * !ses->kbdprefix.rep);
-		else if (ev->x == ']') rep_ev(ses, fd, hscroll, 1 + 7 * !ses->kbdprefix.rep);
-		/*else if (upcase(ev->x) == 'Y' && ev->y == KBD_CTRL) rep_ev(ses, fd, scroll, -1);
-		else if (upcase(ev->x) == 'E' && ev->y == KBD_CTRL) rep_ev(ses, fd, scroll, 1);*/
-		else if (ev->x == KBD_HOME) rep_ev(ses, fd, home, 0);
-		else if (ev->x == KBD_END) rep_ev(ses, fd, x_end, 0);
-		else if (ev->x == KBD_RIGHT || ev->x == KBD_ENTER) x = enter(ses, fd, 0);
-		else if (upcase(ev->x) == 'D' && !(ev->y & KBD_ALT)) {
-			if (!anonymous) frm_download(ses, fd);
-		} else if (ev->x == '/') search_dlg(ses, fd, 0);
-		else if (ev->x == '?') search_back_dlg(ses, fd, 0);
-		else if (ev->x == 'n' && !(ev->y & KBD_ALT)) find_next(ses, fd, 0);
-		else if (ev->x == 'N' && !(ev->y & KBD_ALT)) find_next_back(ses, fd, 0);
-		else if (upcase(ev->x) == 'F' && !(ev->y & KBD_ALT)) set_frame(ses, fd, 0), x = 2;
-		else if (ev->x >= '1' && ev->x <= '9' && !ev->y) {
-			struct f_data *f_data = fd->f_data;
-			int nl, lnl;
-			unsigned char d[2];
-			d[0] = ev->x;
-			d[1] = 0;
-			nl = f_data->nlinks, lnl = 1;
-			while (nl) nl /= 10, lnl++;
-			if (lnl > 1) input_field(ses->term, NULL, TEXT(T_GO_TO_LINK), TEXT(T_ENTER_LINK_NUMBER), TEXT(T_OK), TEXT(T_CANCEL), ses, NULL, lnl, d, 1, f_data->nlinks, check_number, (void (*)(void *, unsigned char *)) goto_link_number, NULL);
-		}
-		/*else if (ev->x == 'x') {
-			struct node *node;
-			static int n = -1;
-			int i;
-			fd->xl = -1234;
-			fd->search_word = ses->search_word;
-			draw_doc(ses->term, fd, 1);
-			clear_link(ses->term, fd);
-			n++;
-			i = n;
-			foreachback(node, fd->f_data->nodes) {
-				if (!i--) {
-					int x, y;
-					for (y = 0; y < node->yw; y++) for (x = 0; x < node->xw && x < 1000; x++) {
-						int rx = x + node->x + fd->xp - fd->vs->view_posx;
-						int ry = y + node->y + fd->yp - fd->vs->view_pos;
-						if (rx >= 0 && ry >= 0 && rx < ses->term->x && ry < ses->term->y) {
-							set_color(ses->term, rx, ry, 0x3800);
+		switch (kbd_action(KM_MAIN, ev)) {
+			case ACT_PAGE_DOWN: rep_ev(ses, fd, page_down, 0); break;
+			case ACT_PAGE_UP: rep_ev(ses, fd, page_up, 0); break;
+			case ACT_DOWN: rep_ev(ses, fd, down, 0); break;
+			case ACT_UP: rep_ev(ses, fd, up, 0); break;
+			case ACT_COPY_CLIPBOARD: {
+				char *current_link = print_current_link(ses);
+				set_clipboard_text( current_link );
+				mem_free(current_link);
+				break;
+			}
+			case ACT_SCROLL_UP: rep_ev(ses, fd, scroll, -1 - !ses->kbdprefix.rep); break;
+			case ACT_SCROLL_DOWN: rep_ev(ses, fd, scroll, 1 + !ses->kbdprefix.rep); break;
+			case ACT_SCROLL_LEFT: rep_ev(ses, fd, hscroll, -1 - 7 * !ses->kbdprefix.rep); break;
+			case ACT_SCROLL_RIGHT: rep_ev(ses, fd, hscroll, 1 + 7 * !ses->kbdprefix.rep); break;
+			case ACT_HOME: rep_ev(ses, fd, home, 0); break;
+			case ACT_END:  rep_ev(ses, fd, x_end, 0); break;
+			case ACT_ENTER: x = enter(ses, fd, 0); break;
+			case ACT_DOWNLOAD: if (!anonymous) frm_download(ses, fd); break;
+			case ACT_SEARCH: search_dlg(ses, fd, 0); break;
+			case ACT_SEARCH_BACK: search_back_dlg(ses, fd, 0); break;
+			case ACT_FIND_NEXT: find_next(ses, fd, 0); break;
+			case ACT_FIND_NEXT_BACK: find_next_back(ses, fd, 0); break;
+			case ACT_ZOOM_FRAME: set_frame(ses, fd, 0), x = 2; break;
+			case ACT_VIEW_IMAGE: send_image(ses->term, NULL, ses); break;
+			default:
+				if (ev->x >= '1' && ev->x <= '9' && !ev->y) {
+					struct f_data *f_data = fd->f_data;
+					int nl, lnl;
+					unsigned char d[2];
+					d[0] = ev->x;
+					d[1] = 0;
+					nl = f_data->nlinks, lnl = 1;
+					while (nl) nl /= 10, lnl++;
+					if (lnl > 1) input_field(ses->term, NULL, TEXT(T_GO_TO_LINK), TEXT(T_ENTER_LINK_NUMBER), TEXT(T_OK), TEXT(T_CANCEL), ses, NULL, lnl, d, 1, f_data->nlinks, check_number, (void (*)(void *, unsigned char *)) goto_link_number, NULL);
+				}
+				/*else if (ev->x == 'x') {
+					struct node *node;
+					static int n = -1;
+					int i;
+					fd->xl = -1234;
+					draw_doc(ses->term, fd, 1);
+					clear_link(ses->term, fd);
+					n++;
+					i = n;
+					foreachback(node, fd->f_data->nodes) {
+						if (!i--) {
+							int x, y;
+							for (y = 0; y < node->yw; y++) for (x = 0; x < node->xw && x < 1000; x++) {
+								int rx = x + node->x + fd->xp - fd->vs->view_posx;
+								int ry = y + node->y + fd->yp - fd->vs->view_pos;
+								if (rx >= 0 && ry >= 0 && rx < ses->term->x && ry < ses->term->y) {
+									set_color(ses->term, rx, ry, 0x3800);
+								}
+							}
+							break;
 						}
 					}
-					break;
-				}
-			}
-			if (i >= 0) n = -1;
-			x = 0;
-		}*/
-		else x = 0;
+					if (i >= 0) n = -1;
+					x = 0;
+				}*/
+				else x = 0;
+		}
 	} else if (ev->ev == EV_MOUSE) {
 		struct link *l = choose_mouse_link(fd, ev);
 		if (l) {
@@ -1861,7 +1877,6 @@ int send_to_frame(struct session *ses, struct event *ev)
 		/*internal("document not formatted");*/
 		return 0;
 	}
-	fd->search_word = ses->search_word;
 	r = frame_ev(ses, fd, ev);
 	if (r == 1) {
 		draw_doc(ses->term, fd, 1);
@@ -1894,7 +1909,6 @@ void do_for_frame(struct session *ses, void (*f)(struct session *, struct f_data
 		/*internal("document not formatted");*/
 		return;
 	}
-	fd->search_word = ses->search_word;
 	f(ses, fd, a);
 }
 
@@ -1926,73 +1940,86 @@ void do_mouse_event(struct session *ses, struct event *ev)
 	send_to_frame(ses, &evv);
 }
 
+void head_msg(struct session *ses);
+
 void send_event(struct session *ses, struct event *ev)
 {
 	if (ev->ev == EV_KBD) {
 		if (send_to_frame(ses, ev)) return;
-		if (ev->y & KBD_ALT) {
-			struct window *m;
-			ev->y &= ~KBD_ALT;
-			activate_bfu_technology(ses, -1);
-			m = ses->term->windows.next;
-			m->handler(m, ev, 0);
-			if (ses->term->windows.next == m) {
-				delete_window(m);
-			} else goto x;
-			ev->y |= ~KBD_ALT;
-		}
-		if (ev->x == KBD_ESC || ev->x == KBD_F9) {
-			activate_bfu_technology(ses, -1);
-			goto x;
-		}
-		if (ev->x == KBD_F10) {
-			activate_bfu_technology(ses, 0);
-			goto x;
-		}
-		if (ev->x == KBD_TAB) {
-			next_frame(ses, ev->y ? -1 : 1);
-			draw_formatted(ses);
-			/*draw_frames(ses);
-			print_screen_status(ses);
-			redraw_from_window(ses->win);*/
-		}
-		if (ev->x == KBD_LEFT) {
-			back(ses, NULL, 0);
-			goto x;
-		}
-		if (upcase(ev->x) == 'R' && ev->y == KBD_CTRL) {
-			reload(ses, -1);
-			goto x;
-		}
-		if (ev->x == 'g' && !ev->y) {
-			quak:
-			dialog_goto_url(ses,"");
-			goto x;
-		}
-		if (ev->x == 'G' && !ev->y) {
-			if (list_empty(ses->history)) goto quak;
-			dialog_goto_url(ses, cur_loc(ses)->vs.url);
-			goto x;
-		}
-		if (upcase(ev->x) == 'A') {
-			if (!anonymous) launch_bm_add_doc_dialog(ses->term, NULL, ses);
-			goto x;
-		}
-		if (upcase(ev->x) == 'S') {
-			if (!anonymous) menu_bookmark_manager(ses->term, NULL, ses);
-			goto x;
-		}
-		if ((upcase(ev->x) == 'Q' && !ev->y) || ev->x == KBD_CTRL_C) {
-			exit_prog(ses->term, (void *)(ev->x == KBD_CTRL_C), ses);
-			goto x;
-		}
-		if (ev->x == '=') {
-			state_msg(ses);
-			goto x;
-		}
-		if (ev->x == '\\') {
-			toggle(ses, ses->screen, 0);
-			goto x;
+		switch (kbd_action(KM_MAIN, ev)) {
+			case ACT_MENU:
+				activate_bfu_technology(ses, -1);
+				goto x;
+			case ACT_FILE_MENU:
+				activate_bfu_technology(ses, 0);
+				goto x;
+			case ACT_NEXT_FRAME:
+				next_frame(ses, 1);
+				draw_formatted(ses);
+				/*draw_frames(ses);
+				  print_screen_status(ses);
+				  redraw_from_window(ses->win);*/
+				goto x;
+			case ACT_PREVIOUS_FRAME:
+				next_frame(ses, -1);
+				draw_formatted(ses);
+				goto x;
+			case ACT_BACK:
+				back(ses, NULL, 0);
+				goto x;
+			case ACT_RELOAD:
+				reload(ses, -1);
+				goto x;
+			case ACT_GOTO_URL:
+		  		quak:
+				dialog_goto_url(ses,"");
+				goto x;
+			case ACT_GOTO_URL_CURRENT: {
+				unsigned char *s;
+				if (list_empty(ses->history)) goto quak;
+				s = stracpy(cur_loc(ses)->vs.url);
+				if (strchr(s, POST_CHAR)) *strchr(s, POST_CHAR) = 0;
+				dialog_goto_url(ses, s);
+				mem_free(s);
+				goto x;
+			}
+			case ACT_ADD_BOOKMARK:
+				if (!anonymous) launch_bm_add_doc_dialog(ses->term, NULL, ses);
+				goto x;
+			case ACT_BOOKMARK_MANAGER:
+				if (!anonymous) menu_bookmark_manager(ses->term, NULL, ses);
+				goto x;
+			case ACT_QUIT:
+		  		quit:
+				exit_prog(ses->term, (void *)(ev->x == KBD_CTRL_C), ses);
+				goto x;
+			case ACT_DOCUMENT_INFO:
+				state_msg(ses);
+				goto x;
+			case ACT_HEADER_INFO:
+				head_msg(ses);
+				goto x;
+			case ACT_TOGGLE_DISPLAY_IMAGES:
+				ses->ds.images = !ses->ds.images;
+				html_interpret(ses);
+				draw_formatted(ses);
+				goto x;
+			case ACT_TOGGLE_HTML_PLAIN:
+				toggle(ses, ses->screen, 0);
+				goto x;
+			default:
+				if (ev->x == KBD_CTRL_C) goto quit;
+				if (ev->y & KBD_ALT) {
+					struct window *m;
+					ev->y &= ~KBD_ALT;
+					activate_bfu_technology(ses, -1);
+					m = ses->term->windows.next;
+					m->handler(m, ev, 0);
+					if (ses->term->windows.next == m) {
+						delete_window(m);
+					} else goto x;
+					ev->y |= ~KBD_ALT;
+				}
 		}
 	}
 	if (ev->ev == EV_MOUSE) {
@@ -2018,9 +2045,12 @@ void send_enter(struct terminal *term, void *xxx, struct session *ses)
 
 void frm_download(struct session *ses, struct f_data_c *fd)
 {
+	struct link *link;
 	if (fd->vs->current_link == -1) return;
-	if (ses->dn_url) mem_free(ses->dn_url);
-	if ((ses->dn_url = get_link_url(ses, fd, &fd->f_data->links[fd->vs->current_link]))) {
+	if (ses->dn_url) mem_free(ses->dn_url), ses->dn_url = NULL;
+	link = &fd->f_data->links[fd->vs->current_link];
+	if (link->type != L_LINK && link->type != L_BUTTON) return;
+	if ((ses->dn_url = get_link_url(ses, fd, link))) {
 		if (!casecmp(ses->dn_url, "MAP@", 4)) {
 			mem_free(ses->dn_url);
 			ses->dn_url = NULL;
@@ -2370,6 +2400,13 @@ void loc_msg(struct terminal *term, struct location *lo, struct f_data_c *frame)
 			add_to_str(&s, &l, ": ");
 			add_to_str(&s, &l, ce->last_modified);
 		}
+#ifdef HAVE_SSL
+		if (ce->ssl_info) {
+			add_to_str(&s, &l, "\n");
+			add_to_str(&s, &l, "SSL cipher: ");
+			add_to_str(&s, &l, ce->ssl_info);
+		}
+#endif
 	}
 	if ((a = print_current_linkx(frame, term))) {
 		add_to_str(&s, &l, "\n\n");
@@ -2388,3 +2425,22 @@ void state_msg(struct session *ses)
 	else loc_msg(ses->term, cur_loc(ses), current_frame(ses));
 }
 
+void head_msg(struct session *ses)
+{
+	struct cache_entry *ce;
+	unsigned char *s, *ss;
+	int len;
+	if (list_empty(ses->history)) {
+		msg_box(ses->term, NULL, TEXT(T_HEADER_INFO), AL_LEFT, TEXT(T_YOU_ARE_NOWHERE), NULL, 1, TEXT(T_OK), NULL, B_ENTER | B_ESC);
+		return;
+	}
+	if (!find_in_cache(cur_loc(ses)->vs.url, &ce)) {
+		ss = s = stracpy(ce->head);
+		len = strlen(s) - 1;
+		if (len > 0) {
+			while (ss = strstr(s, "\r\n")) memmove(ss, ss + 1, strlen(ss));
+			while (*s && s[strlen(s) - 1] == '\n') s[strlen(s) - 1] = 0;
+		}
+		msg_box(ses->term, getml(s, NULL), TEXT(T_HEADER_INFO), AL_LEFT, s, NULL, 1, TEXT(T_OK), NULL, B_ENTER | B_ESC);
+	}
+}
